
// includes
#include "../input/Mouse.hpp"
#include "../platform/System.hpp"
#include "../platform/Window.hpp"

#include "GUIManager.hpp"

using namespace bleifrei::platform;
using namespace bleifrei::input;
using namespace bleifrei;

namespace bleifrei {
	namespace gui {

		static int buttonName[] =
		{
			MouseButtonLeft,
			MouseButtonRight,
			MouseButtonMiddle
		};

		GUIManager::GUIManager(void)
		{
			container.setFlags(GUIComponentFlagStretchHorizontal | GUIComponentFlagStretchVertical);
			resize(Window::getInstance().getWidth(), Window::getInstance().getHeight());
			mouse_button[0] = mouse_button[1] = mouse_button[2] = 0;
			mouse = &Mouse::getInstance();
		}

		GUIManager::~GUIManager(void)
		{
			container.cleanup();
		}

		void GUIManager::render(void)
		{
			container.paint(0, 0);
		}

		void GUIManager::update(void)
		{
			mouse->getPosition(mouse_x, mouse_y);

			for (int button = 0; button < 3; ++button)
			{
				if (mouse->isDown(buttonName[button])) {
					if (!mouse_button[button]) {
						mouse_button[button] = true;
						container.mouseDown(buttonName[button], mouse_x, mouse_y);
					}
				}
				else {
					if (mouse_button[button]) {
						mouse_button[button] = false;
						container.mouseUp(buttonName[button], mouse_x, mouse_y);
					}
				}
			}

			if (mouse_x != mouse_old_x || mouse_y != mouse_old_y)
			{
				if (mouse_button[0] || mouse_button[1] || mouse_button[2]) {
					container.motion(mouse_x, mouse_y);
				}
				else {
					container.passiveMotion(mouse_x, mouse_y);
				}
			}

			mouse_old_x = mouse_x;
			mouse_old_y = mouse_y;

			container.update();
		}

		void GUIManager::resize(int width, int height)
		{
			container.resize(width, height);
		}

		void GUIManager::keyTyped(char ch)
		{
			container.key(ch, mouse_x, mouse_y);
		}

		void GUIManager::keyDown(int key)
		{
			container.special(key, mouse_x, mouse_y);
		}

	}
}