
#ifndef _include_bleifrei_scene_Material_hpp_
#define _include_bleifrei_scene_Material_hpp_ 


// includes
#include <iostream>
#include <string>

#include "../math/Matrix.hpp"

namespace bleifrei {
	namespace gfx {
		class Effect;
		class Technique;
	}
	namespace tex {
		class Texture;
	}
	namespace scene {

		class Material
		{
		public:
			Material(void);
			Material(const Material &material);
			
			~Material(void);

			void setName(const std::string &name)					{ m_name = name;					}
			void setDiffuseMap1(tex::Texture *diffuseMap);
			void setDiffuseMap2(tex::Texture *diffuseMap);
			void setOpacityMap(tex::Texture *opacityMap);
			void setReflectionMap(tex::Texture *reflectionMap);
			void setBumpMap(tex::Texture *bumpMap);
			void setSpecularMap(tex::Texture *specularMap);
			void setAmbientColor(const math::Vector3f &ambient)		{ m_ambient = ambient;				}
			void setDiffuseColor(const math::Vector3f &diffuse)		{ m_diffuse = diffuse;				}
			void setSpecularColor(const math::Vector3f &specular)	{ m_specular = specular;			}
			void setShininess(float shininess)						{ m_shininess = shininess;			}
			void setTransparency(float transparency)				{ m_transparency = transparency;	}

			const std::string &getName(void) const					{ return m_name;					}
			tex::Texture *getDiffuseMap1(void) const				{ return m_diffuseMap1;				}
			tex::Texture *getDiffuseMap2(void) const				{ return m_diffuseMap2;				}
			tex::Texture *getOpacityMap(void) const					{ return m_opacityMap;				}
			tex::Texture *getReflectionMap(void) const				{ return m_reflectionMap;			}
			tex::Texture *getBumpMap(void) const					{ return m_bumpMap;					}
			tex::Texture *getSpecularMap(void) const				{ return m_specularMap;				}
			const math::Vector3f &getAmbientColor(void) const		{ return m_ambient;					}
			const math::Vector3f &getDiffuseColor(void) const		{ return m_diffuse;					}
			const math::Vector3f &getSpecularColor(void) const		{ return m_specular;				}
			float getShininess(void) const							{ return m_shininess;				}
			float getTransparency(void) const						{ return m_transparency;			}

			void read(std::istream &is);
			void write(std::ostream &os);

		private:
			std::string m_name;

			tex::Texture *m_diffuseMap1;
			tex::Texture *m_diffuseMap2;
			tex::Texture *m_opacityMap;
			tex::Texture *m_reflectionMap;
			tex::Texture *m_bumpMap;
			tex::Texture *m_specularMap;

			math::Vector3f m_ambient;
			math::Vector3f m_diffuse;
			math::Vector3f m_specular;

			float m_shininess;
			float m_transparency;
		};

	}
}


#endif //_include_bleifrei_scene_Material_hpp_