
//-------------------------------------------------------------------------------------------------
// carscene-2.fx
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
// Parameters with semantics and default values.
//-------------------------------------------------------------------------------------------------
float4x4 model				: MODEL_MATRIX;
float4x4 viewProjection		: VIEW_PROJECTION_MATRIX;

float3 light0ambient		: LIGHT0_AMBIENT;
float3 light0diffuse		: LIGHT0_DIFFUSE;
float3 light0specular		: LIGHT0_SPECULAR;
float3 light0position		: LIGHT0_POSITION;

float3 materialAmbient		: MATERIAL_AMBIENT;
float3 materialDiffuse		: MATERIAL_DIFFUSE;
float3 materialSpecular		: MATERIAL_SPECULAR;
float materialShininess		: MATERIAL_SHININESS; 

float time					: TIME;
float3 cameraPosition		: CAMERA_POSITION;

//-------------------------------------------------------------------------------------------------
// Input/output structures used by shaders. Each member has a standard semantic defined by the
// Cg shading language.
//-------------------------------------------------------------------------------------------------
struct VS_INPUT
{
	float4 position			: POSITION;
	float3 normal			: NORMAL;
	float2 tex0				: TEXCOORD0;
};

struct VS_OUTPUT
{
    float4 position			: POSITION;
    float2 tex0				: TEXCOORD0;
    float3 worldNormal		: TEXCOORD1;
    float3 worldPosition	: TEXCOORD2;
};

struct VS_SIMPLE_OUTPUT
{
    float4 position			: POSITION;
    float2 tex0				: TEXCOORD0;
};

//-------------------------------------------------------------------------------------------------
// samplerDiffuseBilin
//		Texture sampler for the diffuse texture map.
//-------------------------------------------------------------------------------------------------
sampler2D samplerDiffuseBilin : DIFFUSE_MAP = sampler_state
{
    MinFilter = Linear;
    MagFilter = Linear;
};

// $TASK 2A: Create a Sampler for the Environment Map

//-------------------------------------------------------------------------------------------------
// cubeMap
//		This Cg-function samples the environment map.
//-------------------------------------------------------------------------------------------------
float3 cubeMap(float3 cubeTexcoord, uniform samplerCUBE samplerEnvironmentBilin)
{
	// $TASK 2B: Do a Cubemap Texture Lookup
    return 0;
}

//-------------------------------------------------------------------------------------------------
// calculateReflectionVector
//		This Cg-function calculates the reflection of the vector direction in the plane represented by normal.
//-------------------------------------------------------------------------------------------------
float3 calculateReflectionVector(float3 direction, float3 normal)
{
	// $TASK 2C: Calculate the Reflection Vector
    return normal;
}

//-------------------------------------------------------------------------------------------------
// calculateAmbient
//		This Cg-function calculates the ambient lighting term of light 0.
//-------------------------------------------------------------------------------------------------
float3 calculateAmbient(float3 light0ambient, float3 materialAmbient)
{
    return light0ambient * materialAmbient;
}

//-------------------------------------------------------------------------------------------------
// calculateDiffuse
//		This Cg-function calculates the diffuse lighting term of light 0.
//-------------------------------------------------------------------------------------------------
float3 calculateDiffuse(float3 light0diffuse, float3 materialDiffuse, float3 normal, float3 directionToLight)
{
    return light0diffuse * materialDiffuse * max(0, dot(normal, directionToLight));
}

//-------------------------------------------------------------------------------------------------
// calculateSpecular
//		This Cg-function calculates the specular lighting term of light 0.
//-------------------------------------------------------------------------------------------------
float3 calculateSpecular(float3 light0specular, float3 materialSpecular, float materialShininess, float3 normal, const float3 directionFromLight, const float3 directionToEye)
{
	// calculate reflection vector
    float3 r = normalize(reflect(directionFromLight, normal));

    return light0specular * materialSpecular * pow(max(0, dot(r, directionToEye)), materialShininess / 4);
}

//-------------------------------------------------------------------------------------------------
// vsSimple
//		Vertex shader used for the shadowed plane in the scene. Transforms the vertices to screen
//		space and outputs unmodified texture coordinates.
//-------------------------------------------------------------------------------------------------
VS_SIMPLE_OUTPUT vsSimple(VS_INPUT indata, uniform float4x4 model, uniform float4x4 viewProjection)
{
	VS_SIMPLE_OUTPUT outdata = (VS_SIMPLE_OUTPUT)0;
	
	// transform the position
    float4 position = mul(model, indata.position);						// position (world space)
    outdata.position  = mul(viewProjection, position);					// position (projected)
    
	outdata.tex0 = indata.tex0;
	
	return outdata;
}

//-------------------------------------------------------------------------------------------------
// psSimple
//		Pixel shader used for the shadowed plane in the scene. Samples the diffuse map using
//		texture coordinate set 0.
//-------------------------------------------------------------------------------------------------
float4 psSimple(VS_SIMPLE_OUTPUT indata, uniform sampler2D samplerDiffuseBilin) : COLOR
{
    return tex2D(samplerDiffuseBilin, indata.tex0);
}

//-------------------------------------------------------------------------------------------------
// vsHemi
//		Vertex shader used for the car. Transforms the position and normal of the vertices and
//		calculates the hemispheric lighting term by evaluation the function 'calculateHemisphere'.
//-------------------------------------------------------------------------------------------------
VS_OUTPUT vsEnvironment(VS_INPUT indata, uniform float3 cameraPosition, uniform float time, uniform float4x4 model, uniform float4x4 viewProjection, uniform float3 light0position, uniform float3 light0ambient, uniform float3 light0diffuse, uniform float3 light0specular, uniform float3 materialAmbient, uniform float3 materialDiffuse, uniform float3 materialSpecular, uniform float3 materialShininess)
{
	VS_OUTPUT outdata = (VS_OUTPUT)0;
	
	// transform the position
    float3 position = mul(model, indata.position).xyz;						// position (world space)
    outdata.worldPosition = position;
    outdata.position  = mul(viewProjection, float4(outdata.worldPosition, 1));		// position (projected)

	// transform the normal
    outdata.worldNormal = normalize(mul((float3x3)model, indata.normal));		// normal (world space)

	// Pass through texture coordinate    
	outdata.tex0 = indata.tex0;
    
    return outdata;
}

//-------------------------------------------------------------------------------------------------
// psEnvironment
//		Pixel shader used for the car. Outputs the lighting value calculated in the vertex shader
//		and interpolated across the triangle by the GPU. 
//-------------------------------------------------------------------------------------------------
float4 psEnvironment(VS_OUTPUT indata, uniform sampler2D samplerDiffuseBilin, uniform samplerCUBE samplerEnvironmentBilin, uniform float3 cameraPosition, uniform float3 light0position, uniform float3 light0ambient, uniform float3 light0diffuse, uniform float3 light0specular, uniform float3 materialAmbient, uniform float3 materialDiffuse, uniform float3 materialSpecular, uniform float3 materialShininess) : COLOR
{
    float4 outColor;
    
    
	// calculate vectors used for lighting
    float3 directionToEye = normalize(cameraPosition - indata.worldPosition);
    float3 directionToLight = normalize(light0position.xyz - indata.worldPosition);
    float3 normal = normalize(indata.worldNormal);
    float3 reflectionDir = calculateReflectionVector(-directionToEye, normal);

    // calculate the lighting terms
    float3 diffuse = calculateAmbient(light0ambient, materialAmbient)
				   + calculateDiffuse(light0diffuse, materialDiffuse, normal, directionToLight); 
    
    float3 specular = calculateSpecular(light0specular, materialSpecular, materialShininess, normal, -directionToLight, directionToEye);

    float3 environment = cubeMap(reflectionDir, samplerEnvironmentBilin);
    
    float4 tex = tex2D(samplerDiffuseBilin, indata.tex0);

    float R0 = 0.05f;
    // $TASK 2E: Add a Fresnel Term
	float fresnel = saturate(0.15f);

	// Glossiness is the inverse of the alpha value
	float glossiness = (1 - tex.w) * fresnel;
	
	// $TASK 2D: Adjust Reflectivity
	outColor.rgb = diffuse * tex.xyz + specular + environment;
	
	// Set alpha value for blending
	// $TASK 2E: Add a Fresnel Term
    outColor.a = tex.a;
    
	return outColor;
}
//-------------------------------------------------------------------------------------------------
// simple
//		Technique used for the shadowed plane in the scene.
//-------------------------------------------------------------------------------------------------
technique simple
{
	pass p0
	{
		BlendEnable = false;	
		CullFace = Back;
		DepthMask = true;
		DepthTestEnable = true;
		VertexProgram = compile arbvp1 vsSimple(model, viewProjection);
		FragmentProgram = compile arbfp1 psSimple(samplerDiffuseBilin);
	}
}

//-------------------------------------------------------------------------------------------------
// opaque
//		Technique used for the non-transparent triangles.
//-------------------------------------------------------------------------------------------------
technique opaque
{
	pass p0
	{
		BlendEnable = false;	
		CullFace = Back;
		DepthMask = true;
		DepthTestEnable = true;
		VertexProgram = compile arbvp1 vsEnvironment(cameraPosition, time, model, viewProjection, light0position, light0ambient, light0diffuse, light0specular, materialAmbient, materialDiffuse, materialSpecular, materialShininess);
		FragmentProgram = compile arbfp1 psEnvironment(samplerDiffuseBilin, samplerEnvironmentBilin, cameraPosition, light0position, light0ambient, light0diffuse, light0specular, materialAmbient, materialDiffuse, materialSpecular, materialShininess);
	}
}

//-------------------------------------------------------------------------------------------------
// transparent
//		Technique used for the transparent triangles.
//-------------------------------------------------------------------------------------------------
technique transparent
{
	pass p0
	{
		BlendEnable = true;
		BlendFunc = int2(SrcAlpha, OneMinusSrcAlpha);
		CullFaceEnable = true;
		CullFace = Back;
		DepthMask = true;
		DepthTestEnable = true;
		VertexProgram = compile arbvp1 vsEnvironment(cameraPosition, time, model, viewProjection, light0position, light0ambient, light0diffuse, light0specular, materialAmbient, materialDiffuse, materialSpecular, materialShininess);
		FragmentProgram = compile arbfp1 psEnvironment(samplerDiffuseBilin, samplerEnvironmentBilin, cameraPosition, light0position, light0ambient, light0diffuse, light0specular, materialAmbient, materialDiffuse, materialSpecular, materialShininess);
	}
}
